# 1 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright (C) 2000 - 2017  CP2K developers group                                               !
!--------------------------------------------------------------------------------------------------!

! **************************************************************************************************
!> \brief Utility routines for the memory handling.
!>        Uses stop_memory to handle exceptions
!> \par History
!>      none
!> \author Matthias Krack (25.06.1999)
! **************************************************************************************************
MODULE memory_utilities

   USE kinds,                           ONLY: default_path_length,&
                                              default_string_length,&
                                              dp,&
                                              dp_size,&
                                              int_8,&
                                              int_8_size,&
                                              int_size
   USE termination,                     ONLY: stop_memory
#include "../base/base_uses.f90"

   IMPLICIT NONE

   PRIVATE

   CHARACTER(len=*), PARAMETER, PRIVATE :: moduleN = 'memory_utilities'

   PUBLIC :: reallocate

   INTERFACE reallocate
      MODULE PROCEDURE reallocate_c1, reallocate_c2, reallocate_c3, reallocate_c4, &
         reallocate_i1, reallocate_i2, reallocate_i3, reallocate_i4, &
         reallocate_r1, reallocate_r2, reallocate_r3, reallocate_r4, &
         reallocate_r5, reallocate_s1, reallocate_l1, reallocate_8i1, &
         reallocate_8i2
   END INTERFACE

CONTAINS

# 47 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
# 91 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

! **************************************************************************************************
!> \brief (Re)Allocate a complex vector with a new dimension
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \par History
!>      none
!> \author Matthias Krack (28.11.2005,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_c1(p, lb1_new, ub1_new)

      COMPLEX(KIND=dp), DIMENSION(:), POINTER            :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_c1', routineP = moduleN//':'//routineN
      COMPLEX(KIND=dp), PARAMETER                        :: zero = (0.0_dp, 0.0_dp)
      INTEGER, PARAMETER                                 :: t_size = 2*dp_size

      COMPLEX(KIND=dp), ALLOCATABLE, DIMENSION(:)        :: work


# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1 ), STAT=istat)
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1))
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1) = p(lb1:ub1)
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new),STAT=istat)
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1))
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1) = work(lb1:ub1)
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 112 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_c1

! **************************************************************************************************
!> \brief (Re)Allocate a complex array 2D with a new dimension
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \par History
!>      none
!> \author Matthias Krack (28.11.2005,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_c2(p, lb1_new, ub1_new, lb2_new, ub2_new)

      COMPLEX(KIND=dp), DIMENSION(:, :), POINTER         :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_c2', routineP = moduleN//':'//routineN
      COMPLEX(KIND=dp), PARAMETER                        :: zero = (0.0_dp, 0.0_dp)
      INTEGER, PARAMETER                                 :: t_size = 2*dp_size

      COMPLEX(KIND=dp), ALLOCATABLE, DIMENSION(:, :)     :: work


# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb2, lb2_old, ub2, ub2_old
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2_old = LBOUND(p,2)
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2_old = UBOUND(p,2)
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2 = MAX(lb2_new,lb2_old)
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2 = MIN(ub2_new,ub2_old)
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1,lb2:ub2 ), STAT=istat)
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1)*(ub2-lb2+1))
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1,lb2:ub2) = p(lb1:ub1,lb2:ub2)
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new),STAT=istat)
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1)*(ub2_new-lb2_new+1))
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1,lb2:ub2) = work(lb1:ub1,lb2:ub2)
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 138 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_c2

! **************************************************************************************************
!> \brief (Re)Allocate a complex array 3D with a new dimension
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \param lb3_new ...
!> \param ub3_new ...
!> \par History
!>      none
!> \author Matthias Krack (28.11.2005,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_c3(p, lb1_new, ub1_new, lb2_new, ub2_new, lb3_new, ub3_new)

      COMPLEX(KIND=dp), DIMENSION(:, :, :), POINTER      :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new, &
                                                            lb3_new, ub3_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_c3', routineP = moduleN//':'//routineN
      COMPLEX(KIND=dp), PARAMETER                        :: zero = (0.0_dp, 0.0_dp)
      INTEGER, PARAMETER                                 :: t_size = 2*dp_size

      COMPLEX(KIND=dp), ALLOCATABLE, DIMENSION(:, :, :)  :: work


# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb2, lb2_old, ub2, ub2_old
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb3, lb3_old, ub3, ub3_old
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2_old = LBOUND(p,2)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2_old = UBOUND(p,2)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2 = MAX(lb2_new,lb2_old)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2 = MIN(ub2_new,ub2_old)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb3_old = LBOUND(p,3)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub3_old = UBOUND(p,3)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb3 = MAX(lb3_new,lb3_old)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub3 = MIN(ub3_new,ub3_old)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1,lb2:ub2,lb3:ub3 ), STAT=istat)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1)*(ub2-lb2+1)*(ub3-lb3+1))
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1,lb2:ub2,lb3:ub3) = p(lb1:ub1,lb2:ub2,lb3:ub3)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new),STAT=istat)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1)*(ub2_new-lb2_new+1)*(ub3_new-lb3_new+1))
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1,lb2:ub2,lb3:ub3) = work(lb1:ub1,lb2:ub2,lb3:ub3)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 167 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_c3

! **************************************************************************************************
!> \brief (Re)Allocate a complex array 4D with a new dimension
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \param lb3_new ...
!> \param ub3_new ...
!> \param lb4_new ...
!> \param ub4_new ...
!> \par History
!>      none
!> \author Matthias Krack (28.11.2005,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_c4(p, lb1_new, ub1_new, lb2_new, ub2_new, lb3_new, ub3_new, &
                            lb4_new, ub4_new)

      COMPLEX(KIND=dp), DIMENSION(:, :, :, :), POINTER   :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new, &
                                                            lb3_new, ub3_new, lb4_new, ub4_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_c4', routineP = moduleN//':'//routineN
      COMPLEX(KIND=dp), PARAMETER                        :: zero = (0.0_dp, 0.0_dp)
      INTEGER, PARAMETER                                 :: t_size = 2*dp_size

      COMPLEX(KIND=dp), ALLOCATABLE, &
         DIMENSION(:, :, :, :)                           :: work


# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb2, lb2_old, ub2, ub2_old
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb3, lb3_old, ub3, ub3_old
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb4, lb4_old, ub4, ub4_old
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2_old = LBOUND(p,2)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2_old = UBOUND(p,2)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2 = MAX(lb2_new,lb2_old)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2 = MIN(ub2_new,ub2_old)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb3_old = LBOUND(p,3)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub3_old = UBOUND(p,3)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb3 = MAX(lb3_new,lb3_old)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub3 = MIN(ub3_new,ub3_old)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb4_old = LBOUND(p,4)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub4_old = UBOUND(p,4)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb4 = MAX(lb4_new,lb4_old)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub4 = MIN(ub4_new,ub4_old)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4 ), STAT=istat)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1)*(ub2-lb2+1)*(ub3-lb3+1)*(ub4-lb4+1))
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4) = p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new,lb4_new:ub4_new),STAT=istat)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1)*(ub2_new-lb2_new+1)*(ub3_new-lb3_new+1)*(ub4_new-lb4_new+1))
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4) = work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 200 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_c4

! **************************************************************************************************
!> \brief (Re)Allocate an integer vector with a new dimension.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_i1(p, lb1_new, ub1_new)

      INTEGER, DIMENSION(:), POINTER                     :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_i1', routineP = moduleN//':'//routineN
      INTEGER, PARAMETER                                 :: t_size = int_size, zero = 0

      INTEGER, ALLOCATABLE, DIMENSION(:)                 :: work


# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1 ), STAT=istat)
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1))
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1) = p(lb1:ub1)
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new),STAT=istat)
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1))
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1) = work(lb1:ub1)
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 223 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_i1

! **************************************************************************************************
!> \brief (Re)Allocate an integer array with a new dimension.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_i2(p, lb1_new, ub1_new, lb2_new, ub2_new)

      INTEGER, DIMENSION(:, :), POINTER                  :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_i2', routineP = moduleN//':'//routineN
      INTEGER, PARAMETER                                 :: t_size = int_size, zero = 0

      INTEGER, ALLOCATABLE, DIMENSION(:, :)              :: work


# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb2, lb2_old, ub2, ub2_old
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2_old = LBOUND(p,2)
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2_old = UBOUND(p,2)
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2 = MAX(lb2_new,lb2_old)
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2 = MIN(ub2_new,ub2_old)
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1,lb2:ub2 ), STAT=istat)
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1)*(ub2-lb2+1))
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1,lb2:ub2) = p(lb1:ub1,lb2:ub2)
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new),STAT=istat)
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1)*(ub2_new-lb2_new+1))
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1,lb2:ub2) = work(lb1:ub1,lb2:ub2)
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 248 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_i2

! **************************************************************************************************
!> \brief (Re)Allocate an integer array 3D with a new dimension.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \param lb3_new ...
!> \param ub3_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_i3(p, lb1_new, ub1_new, lb2_new, ub2_new, lb3_new, ub3_new)

      INTEGER, DIMENSION(:, :, :), POINTER               :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new, &
                                                            lb3_new, ub3_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_i3', routineP = moduleN//':'//routineN
      INTEGER, PARAMETER                                 :: t_size = int_size, zero = 0

      INTEGER, ALLOCATABLE, DIMENSION(:, :, :)           :: work


# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb2, lb2_old, ub2, ub2_old
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb3, lb3_old, ub3, ub3_old
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2_old = LBOUND(p,2)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2_old = UBOUND(p,2)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2 = MAX(lb2_new,lb2_old)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2 = MIN(ub2_new,ub2_old)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb3_old = LBOUND(p,3)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub3_old = UBOUND(p,3)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb3 = MAX(lb3_new,lb3_old)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub3 = MIN(ub3_new,ub3_old)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1,lb2:ub2,lb3:ub3 ), STAT=istat)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1)*(ub2-lb2+1)*(ub3-lb3+1))
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1,lb2:ub2,lb3:ub3) = p(lb1:ub1,lb2:ub2,lb3:ub3)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new),STAT=istat)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1)*(ub2_new-lb2_new+1)*(ub3_new-lb3_new+1))
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1,lb2:ub2,lb3:ub3) = work(lb1:ub1,lb2:ub2,lb3:ub3)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 276 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_i3

! **************************************************************************************************
!> \brief (Re)Allocate an integer array 3D with a new dimension.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \param lb3_new ...
!> \param ub3_new ...
!> \param lb4_new ...
!> \param ub4_new ...
!> \par History
!>      none
!> \author Matthias Krack (04.10.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_i4(p, lb1_new, ub1_new, lb2_new, ub2_new, lb3_new, ub3_new, &
                            lb4_new, ub4_new)

      INTEGER, DIMENSION(:, :, :, :), POINTER            :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new, &
                                                            lb3_new, ub3_new, lb4_new, ub4_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_i4', routineP = moduleN//':'//routineN
      INTEGER, PARAMETER                                 :: t_size = int_size, zero = 0

      INTEGER, ALLOCATABLE, DIMENSION(:, :, :, :)        :: work


# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb2, lb2_old, ub2, ub2_old
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb3, lb3_old, ub3, ub3_old
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb4, lb4_old, ub4, ub4_old
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2_old = LBOUND(p,2)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2_old = UBOUND(p,2)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2 = MAX(lb2_new,lb2_old)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2 = MIN(ub2_new,ub2_old)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb3_old = LBOUND(p,3)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub3_old = UBOUND(p,3)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb3 = MAX(lb3_new,lb3_old)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub3 = MIN(ub3_new,ub3_old)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb4_old = LBOUND(p,4)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub4_old = UBOUND(p,4)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb4 = MAX(lb4_new,lb4_old)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub4 = MIN(ub4_new,ub4_old)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4 ), STAT=istat)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1)*(ub2-lb2+1)*(ub3-lb3+1)*(ub4-lb4+1))
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4) = p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new,lb4_new:ub4_new),STAT=istat)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1)*(ub2_new-lb2_new+1)*(ub3_new-lb3_new+1)*(ub4_new-lb4_new+1))
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4) = work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 307 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_i4

! **************************************************************************************************
!> \brief (Re)Allocate an integer (int_8) vector with a new dimension.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_8i1(p, lb1_new, ub1_new)

      INTEGER(KIND=int_8), DIMENSION(:), POINTER         :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_8i1', routineP = moduleN//':'//routineN
      INTEGER(KIND=int_8), PARAMETER                     :: zero = 0
      INTEGER, PARAMETER                                 :: t_size = int_8_size

      INTEGER(KIND=int_8), ALLOCATABLE, DIMENSION(:)     :: work


# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1 ), STAT=istat)
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1))
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1) = p(lb1:ub1)
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new),STAT=istat)
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1))
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1) = work(lb1:ub1)
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 331 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_8i1

! **************************************************************************************************
!> \brief (Re)Allocate an integer (int_8) array with a new dimension.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_8i2(p, lb1_new, ub1_new, lb2_new, ub2_new)

      INTEGER(kind=int_8), DIMENSION(:, :), POINTER      :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_8i2', routineP = moduleN//':'//routineN
      INTEGER(KIND=int_8), PARAMETER                     :: zero = 0
      INTEGER, PARAMETER                                 :: t_size = int_8_size

      INTEGER(KIND=int_8), ALLOCATABLE, DIMENSION(:, :)  :: work


# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb2, lb2_old, ub2, ub2_old
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2_old = LBOUND(p,2)
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2_old = UBOUND(p,2)
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2 = MAX(lb2_new,lb2_old)
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2 = MIN(ub2_new,ub2_old)
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1,lb2:ub2 ), STAT=istat)
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1)*(ub2-lb2+1))
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1,lb2:ub2) = p(lb1:ub1,lb2:ub2)
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new),STAT=istat)
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1)*(ub2_new-lb2_new+1))
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1,lb2:ub2) = work(lb1:ub1,lb2:ub2)
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 357 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_8i2

! **************************************************************************************************
!> \brief (Re)Allocate an real vector with a new dimension.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_r1(p, lb1_new, ub1_new)

      REAL(KIND=dp), DIMENSION(:), POINTER               :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_r1', routineP = moduleN//':'//routineN
      INTEGER, PARAMETER                                 :: t_size = dp_size
      REAL(KIND=dp), PARAMETER                           :: zero = 0.0_dp

      REAL(KIND=dp), ALLOCATABLE, DIMENSION(:)           :: work


# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1 ), STAT=istat)
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1))
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1) = p(lb1:ub1)
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new),STAT=istat)
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1))
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1) = work(lb1:ub1)
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 381 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_r1

! **************************************************************************************************
!> \brief (Re)Allocate an real array with new dimensions.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_r2(p, lb1_new, ub1_new, lb2_new, ub2_new)

      REAL(KIND=dp), DIMENSION(:, :), POINTER            :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_r2', routineP = moduleN//':'//routineN
      INTEGER, PARAMETER                                 :: t_size = dp_size
      REAL(KIND=dp), PARAMETER                           :: zero = 0.0_dp

      REAL(KIND=dp), ALLOCATABLE, DIMENSION(:, :)        :: work


# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb2, lb2_old, ub2, ub2_old
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2_old = LBOUND(p,2)
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2_old = UBOUND(p,2)
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2 = MAX(lb2_new,lb2_old)
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2 = MIN(ub2_new,ub2_old)
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1,lb2:ub2 ), STAT=istat)
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1)*(ub2-lb2+1))
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1,lb2:ub2) = p(lb1:ub1,lb2:ub2)
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new),STAT=istat)
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1)*(ub2_new-lb2_new+1))
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1,lb2:ub2) = work(lb1:ub1,lb2:ub2)
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 407 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_r2

! **************************************************************************************************
!> \brief (Re)Allocate an real array 3D with new dimensions.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \param lb3_new ...
!> \param ub3_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_r3(p, lb1_new, ub1_new, lb2_new, ub2_new, lb3_new, ub3_new)

      REAL(KIND=dp), DIMENSION(:, :, :), POINTER         :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new, &
                                                            lb3_new, ub3_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_r3', routineP = moduleN//':'//routineN
      INTEGER, PARAMETER                                 :: t_size = dp_size
      REAL(KIND=dp), PARAMETER                           :: zero = 0.0_dp

      REAL(KIND=dp), ALLOCATABLE, DIMENSION(:, :, :)     :: work


# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb2, lb2_old, ub2, ub2_old
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb3, lb3_old, ub3, ub3_old
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2_old = LBOUND(p,2)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2_old = UBOUND(p,2)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2 = MAX(lb2_new,lb2_old)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2 = MIN(ub2_new,ub2_old)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb3_old = LBOUND(p,3)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub3_old = UBOUND(p,3)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb3 = MAX(lb3_new,lb3_old)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub3 = MIN(ub3_new,ub3_old)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1,lb2:ub2,lb3:ub3 ), STAT=istat)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1)*(ub2-lb2+1)*(ub3-lb3+1))
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1,lb2:ub2,lb3:ub3) = p(lb1:ub1,lb2:ub2,lb3:ub3)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new),STAT=istat)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1)*(ub2_new-lb2_new+1)*(ub3_new-lb3_new+1))
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1,lb2:ub2,lb3:ub3) = work(lb1:ub1,lb2:ub2,lb3:ub3)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 436 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_r3

! **************************************************************************************************
!> \brief (Re)Allocate an real array 4D with new dimensions.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \param lb3_new ...
!> \param ub3_new ...
!> \param lb4_new ...
!> \param ub4_new ...
!> \par History
!>      none
!> \author Matthias Krack (04.10.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_r4(p, lb1_new, ub1_new, lb2_new, ub2_new, lb3_new, ub3_new, &
                            lb4_new, ub4_new)

      REAL(KIND=dp), DIMENSION(:, :, :, :), POINTER      :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new, &
                                                            lb3_new, ub3_new, lb4_new, ub4_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_r4', routineP = moduleN//':'//routineN
      INTEGER, PARAMETER                                 :: t_size = dp_size
      REAL(KIND=dp), PARAMETER                           :: zero = 0.0_dp

      REAL(KIND=dp), ALLOCATABLE, DIMENSION(:, :, :, :)  :: work


# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb2, lb2_old, ub2, ub2_old
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb3, lb3_old, ub3, ub3_old
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb4, lb4_old, ub4, ub4_old
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2_old = LBOUND(p,2)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2_old = UBOUND(p,2)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2 = MAX(lb2_new,lb2_old)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2 = MIN(ub2_new,ub2_old)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb3_old = LBOUND(p,3)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub3_old = UBOUND(p,3)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb3 = MAX(lb3_new,lb3_old)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub3 = MIN(ub3_new,ub3_old)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb4_old = LBOUND(p,4)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub4_old = UBOUND(p,4)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb4 = MAX(lb4_new,lb4_old)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub4 = MIN(ub4_new,ub4_old)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4 ), STAT=istat)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1)*(ub2-lb2+1)*(ub3-lb3+1)*(ub4-lb4+1))
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4) = p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new,lb4_new:ub4_new),STAT=istat)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1)*(ub2_new-lb2_new+1)*(ub3_new-lb3_new+1)*(ub4_new-lb4_new+1))
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4) = work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 468 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_r4

! **************************************************************************************************
!> \brief (Re)Allocate an real array 5D with new dimensions.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \param lb3_new ...
!> \param ub3_new ...
!> \param lb4_new ...
!> \param ub4_new ...
!> \param lb5_new ...
!> \param ub5_new ...
!> \par History
!>      none
!> \author Matthias Krack (04.10.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_r5(p, lb1_new, ub1_new, lb2_new, ub2_new, lb3_new, ub3_new, &
                            lb4_new, ub4_new, lb5_new, ub5_new)

      REAL(KIND=dp), DIMENSION(:, :, :, :, :), POINTER   :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new, &
                                                            lb3_new, ub3_new, lb4_new, ub4_new, &
                                                            lb5_new, ub5_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_r5', routineP = moduleN//':'//routineN
      INTEGER, PARAMETER                                 :: t_size = dp_size
      REAL(KIND=dp), PARAMETER                           :: zero = 0.0_dp

      REAL(KIND=dp), ALLOCATABLE, &
         DIMENSION(:, :, :, :, :)                        :: work


# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb2, lb2_old, ub2, ub2_old
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb3, lb3_old, ub3, ub3_old
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb4, lb4_old, ub4, ub4_old
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb5, lb5_old, ub5, ub5_old
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2_old = LBOUND(p,2)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2_old = UBOUND(p,2)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb2 = MAX(lb2_new,lb2_old)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub2 = MIN(ub2_new,ub2_old)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb3_old = LBOUND(p,3)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub3_old = UBOUND(p,3)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb3 = MAX(lb3_new,lb3_old)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub3 = MIN(ub3_new,ub3_old)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb4_old = LBOUND(p,4)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub4_old = UBOUND(p,4)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb4 = MAX(lb4_new,lb4_old)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub4 = MIN(ub4_new,ub4_old)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb5_old = LBOUND(p,5)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub5_old = UBOUND(p,5)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb5 = MAX(lb5_new,lb5_old)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub5 = MIN(ub5_new,ub5_old)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4,lb5:ub5 ), STAT=istat)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1)*(ub2-lb2+1)*(ub3-lb3+1)*(ub4-lb4+1)*(ub5-lb5+1))
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4,lb5:ub5) = p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4,lb5:ub5)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new,lb4_new:ub4_new,lb5_new:ub5_new),STAT=istat)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1)*(ub2_new-lb2_new+1)*(ub3_new-lb3_new+1)*(ub4_new-lb4_new+1)*(ub5_new-lb5_new&
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                            &+1))
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4,lb5:ub5) = work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4,lb5:ub5)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 504 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_r5

! **************************************************************************************************
!> \brief (Re)Allocate a logical vector with a new dimension.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_l1(p, lb1_new, ub1_new)

      LOGICAL, DIMENSION(:), POINTER                     :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_l1', routineP = moduleN//':'//routineN
      INTEGER, PARAMETER                                 :: t_size = int_size
      LOGICAL, PARAMETER                                 :: zero = .FALSE.

      LOGICAL, ALLOCATABLE, DIMENSION(:)                 :: work


# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1 ), STAT=istat)
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1))
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1) = p(lb1:ub1)
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new),STAT=istat)
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1))
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1) = work(lb1:ub1)
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 528 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_l1

! **************************************************************************************************
!> \brief (Re)Allocate a vector of string variables
!> \param p_short ...
!> \param lb_new ...
!> \param ub_new ...
!> \param p_long ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
!> \note
!>      (Maybe outdated) This routine doesnt work on SUN/Solaris!!!
!>                       It should probably not be used. (Thomas Chassaing)
! **************************************************************************************************
   SUBROUTINE reallocate_s1(p_short, lb_new, ub_new, p_long)

      CHARACTER(LEN=default_string_length), &
         DIMENSION(:), OPTIONAL, POINTER                 :: p_short
      INTEGER, INTENT(IN)                                :: lb_new, ub_new
      CHARACTER(LEN=default_path_length), DIMENSION(:), &
         OPTIONAL, POINTER                               :: p_long

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_s1', routineP = moduleN//':'//routineN

      IF (PRESENT(p_short)) THEN
         CALL reallocate_ss1(p_short, lb_new, ub_new)
      ELSE IF (PRESENT(p_long)) THEN
         CALL reallocate_ls1(p_long, lb_new, ub_new)
      ELSE
         CPABORT("At least one of the two optional arguments is required")
      END IF

   END SUBROUTINE reallocate_s1

! **************************************************************************************************
!> \brief (Re)Allocate a vector of string variables of default string length.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \par History
!>      none
!> \author Thomas Chassaing
!> \author Matthias Krack (18.07.2002,MK)
!> \note
!>      (Maybe outdated) This routine doesnt work on SUN/Solaris!!!
!>                       It should probably not be used. (Thomas Chassaing)
! **************************************************************************************************
   SUBROUTINE reallocate_ss1(p, lb1_new, ub1_new)

      CHARACTER(LEN=default_string_length), &
         DIMENSION(:), POINTER                           :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_ss1', routineP = moduleN//':'//routineN
      CHARACTER(LEN=default_string_length), PARAMETER    :: zero = ""
      INTEGER, PARAMETER :: t_size = int_size*default_string_length

      CHARACTER(LEN=default_string_length), &
         ALLOCATABLE, DIMENSION(:)                       :: work


# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1 ), STAT=istat)
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1))
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1) = p(lb1:ub1)
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new),STAT=istat)
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1))
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1) = work(lb1:ub1)
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 591 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_ss1

! **************************************************************************************************
!> \brief (Re)Allocate a vector of string variables of default path length.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
!> \note
!>      (Maybe outdated) This routine doesnt work on SUN/Solaris!!!
!>                       It should probably not be used. (Thomas Chassaing)
! **************************************************************************************************
   SUBROUTINE reallocate_ls1(p, lb1_new, ub1_new)

      CHARACTER(LEN=default_path_length), DIMENSION(:), &
         POINTER                                         :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new

      CHARACTER(LEN=*), PARAMETER :: routineN = 'reallocate_ls1', routineP = moduleN//':'//routineN
      CHARACTER(LEN=default_path_length), PARAMETER      :: zero = ""
      INTEGER, PARAMETER :: t_size = int_size*default_path_length

      CHARACTER(LEN=default_path_length), ALLOCATABLE, &
         DIMENSION(:)                                    :: work


# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: istat
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
      INTEGER :: lb1, lb1_old, ub1, ub1_old
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p)) THEN
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1_old = LBOUND(p,1)
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1_old = UBOUND(p,1)
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       lb1 = MAX(lb1_new,lb1_old)
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ub1 = MIN(ub1_new,ub1_old)
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       ALLOCATE (work( lb1:ub1 ), STAT=istat)
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       IF (istat /= 0) THEN
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
          CALL stop_memory(routineN,moduleN,__LINE__,&
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                           "work",t_size*(ub1-lb1+1))
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       END IF
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       work(lb1:ub1) = p(lb1:ub1)
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (p)
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    ALLOCATE (p(lb1_new:ub1_new),STAT=istat)
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (istat /= 0) THEN
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       CALL stop_memory(routineN,moduleN,__LINE__,&
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
                        "p",t_size*(ub1_new-lb1_new+1))
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    p = zero
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"

# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    IF (ASSOCIATED(p).AND.ALLOCATED(work)) THEN
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       p(lb1:ub1) = work(lb1:ub1)
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
       DEALLOCATE (work)
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"
    END IF
# 620 "/builddir/build/BUILD/cp2k-5.1/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_ls1

END MODULE memory_utilities
